//
// Created by zoe on 2/20/23.
//

#include "SentecRainGaugeSensor.h"

static const char *TAG = "SEM404";

bool SEM404::isAnswerFrameEqualToQueryFrame(byte answerFrame[8], byte queryFrame[8]) {
    for (int i = 0; i < 8; i++) {
        if (answerFrame[i] != queryFrame[i]) {
            ESP_LOGD(TAG, "Response: 0x%s", formatBytes(answerFrame, 8).c_str());
            ESP_LOGD(TAG, "Response: 0x%s", formatBytes(queryFrame, 8).c_str());
            ESP_LOGE("TEST", "Error: answerFrame[%d] != ret.writtenQuery[%d]", i, i);
            return false;
        }
    }
    return true;
}

ErrorType SEM404::resetPrecipitation() {
    // clears rainfall rata from the rain gauge
    // delay resetting after the last register reading
    delay(100);
    Serial.println("Resetting precipitation sum");
    auto writeRegisterReturn = writeRegister(0x00, 0x5A);

    // other errors like not connected will be shared when reading out the sensor
    if (!isAnswerFrameEqualToQueryFrame(answerFrame, writeRegisterReturn.writtenQuery)) {
        ESP_LOGE(TAG, "Could not reset precipitation");
        return ErrorType::SEM404_COULD_NOT_RESET_PRECIPITATION;
    }

    return writeRegisterReturn.errorType;
}

void SEM404::resetSensor() {
    // no clue what the difference between this one and the previous one is...
    //   and the manual is not helpful, of course
    delay(100);
    Serial.println("Resetting precipitation sum");
    writeRegister(0x37, 0x03);
    // TODO check response: matches the sent message exactly
}
out_data_rain_gauge SEM404::getInstantaneousPrecipitation() {
    // gets tips of scale since the last reset, i.e. total precipitation (I THINK)
    //   manual says this is current precipitation - is it?
    auto error = readRegister(0, 0x01);
    precipitation = word(answerFrame[3], answerFrame[4]);

    ESP_LOGI("RainGauge", "Precipitation: %s mm", getPrecipitationStr().c_str());
    // resetPrecipitation();
    return {static_cast<float>(precipitation), error};
}
String SEM404::getPrecipitationStr() {
    return getValueStr((float)(precipitation / 10.0));
}

SensorInformation SEM404::getSensorInformation() const {
    return {HardwareName::SEM404, SensorProtocol::RS485};
}
std::list<Message> SEM404::buildMessages() {
    auto messages = std::list<Message>();
    auto data = getInstantaneousPrecipitation();
    Measurement precipitationMeasurement{data.precipitation, address, NO_I2C_ADDRESS, MeasurementType::PRECIPITATION,
                                         data.precipitationError};
    messages.emplace_back(precipitationMeasurement, getSensorInformation(), Time::getInstance().getEpochSeconds());
    return messages;
}
out_data_rain_gauge SEM404::readData() {
    return getInstantaneousPrecipitation();
}
void SEM404::setup() {}
