//
// Created by zoe on 2/20/23.
//

#ifndef CLIENT_CENTRAL_MAST_SENTECRAINGAUGESENSOR_H
#define CLIENT_CENTRAL_MAST_SENTECRAINGAUGESENSOR_H

#include "SentecSensorsRS485.h"
#include <ForteSensor.hpp>
#include <Message.hpp>

struct out_data_rain_gauge {
    float precipitation;
    ErrorType precipitationError;
};

/**
 * @brief Class to read data from the Sentec rain gauge sensor (SEM404)
 * @implements ForteSensor<out_data_rain_gauge>
 * @extends SentecSensorRS485
 */
class SEM404 : public SentecSensorRS485, ForteSensor<out_data_rain_gauge> {
  public:
    using SentecSensorRS485::SentecSensorRS485;
    SEM404() : SentecSensorRS485(0) {}
    // precipitation values [mm]
    unsigned int precipitation = 0; // prcp since reset? (I THINK!!!)

    /**
     * @brief Reset the precipitation value
     * @return ErrorType. If the reset was unsuccessful, this will return ErrorType::SEM404_RESET_PRECIPITATION_ERROR.
     * Otherwise it returns the error given from the device
     */
    ErrorType resetPrecipitation();

    void resetSensor();

    /**
     * @brief Get the instantaneous precipitation value
     * @return out_data_rain_gauge
     */
    out_data_rain_gauge getInstantaneousPrecipitation();

    String getPrecipitationStr();

    void setup() override;
    out_data_rain_gauge readData() override;
    std::list<Message> buildMessages() override;
    [[nodiscard]] SensorInformation getSensorInformation() const override;

  private:
    /**
     * @brief Check if the answer frame is equal to the query frame
     * @param answerFrame The frame returned from the device
     * @param queryFrame The frame sent to the device
     * @return true if the answer frame is equal to the query frame
     */
    bool isAnswerFrameEqualToQueryFrame(byte *answerFrame, byte *queryFrame);
};
#endif // CLIENT_CENTRAL_MAST_SENTECRAINGAUGESENSOR_H
