#ifndef SENTECSENSORS_H
#define SENTECSENSORS_H

#include <Arduino.h>
#include <ErrorTypes.h>
#include <HardwareNames.h>
#include <RS485HardwareSerial.h>
#include <SensorInformation.hpp>
#include <memory>

struct WriteRegisterReturnType {
    ErrorType errorType;
    byte writtenQuery[8];
};

class SentecSensorRS485 {
  public:
    byte address;
    uint8_t serialCommunicationControlPin = 19;
    std::shared_ptr<HardwareSerial> RS485 = RS485HardwareSerial::getInstance().getRS485Serial();
    byte answerFrame[10] = {0};
    // TODO use valid flag to log None
    bool valid = false;

    SentecSensorRS485(byte add);

    SentecSensorRS485(byte add, uint8_t serialControlPin);

    void write(byte queryFrame[], int length);

    String getValueStr(float value);

    String getValueStr(int value);

    ErrorType queryAddress();

    ErrorType readRegister(int registerStartAddress);

    ErrorType readRegister(int registerStartAddress, int registerLength);

    WriteRegisterReturnType writeRegister(int registerAddress, int value);

    ErrorType setAddress(byte add);

    void resetAnswerFrame();

    ErrorType getResponse();

    unsigned int calculateCRC(byte query[], int length);

    void printBytes(byte *data, int length);

    void printBytes(word data);

    std::string formatBytes(byte *data, int length);

    std::string formatBytes(word data);
};

#endif
