//
// Created by zoe on 3/8/23.
//

#ifndef CLIENT_CENTRAL_MAST_RS485HARDWARESERIAL_H
#define CLIENT_CENTRAL_MAST_RS485HARDWARESERIAL_H


#include <HardwareSerial.h>
#include <memory>

/**
 * @brief Singleton class to get the RS485 serial port
 * @example std::shared_ptr<HardwareSerial> RS485 = RS485HardwareSerial::getInstance().getRS485Serial();

 */
class RS485HardwareSerial {
 public:
  static RS485HardwareSerial &getInstance()
  {
    static RS485HardwareSerial instance; // Guaranteed to be destroyed.
    // Instantiated on first use.
    return instance;
  }

    std::shared_ptr<HardwareSerial> getRS485Serial() {
        return RS485Serial;
    }

 private:
  RS485HardwareSerial() {} // Constructor? (the {} brackets) are needed here.

  const std::shared_ptr<HardwareSerial> RS485Serial = std::make_shared<HardwareSerial>(Serial2);


  // C++ 11
  // =======
  // We can use the better technique of deleting the methods
  // we don't want.
 public:
  RS485HardwareSerial(RS485HardwareSerial const &) = delete;
  void operator=(RS485HardwareSerial const &) = delete;

  // Note: Scott Meyers mentions in his Effective Modern
  //       C++ book, that deleted functions should generally
  //       be public as it results in better error messages
  //       due to the compilers behavior to check accessibility
  //       before deleted status

};


#endif //CLIENT_CENTRAL_MAST_RS485HARDWARESERIAL_H
