//
// Created by zoe on 12/19/22.
//

#include "NTPManager.h"

NTPManager::NTPManager(TinyGsmSim7000 &modem) : modem(modem) {}

void NTPManager::syncNTP(const std::string &ntpServer, const std::string &backupNtpServer) {
    // create list of ntp servers
    std::vector<std::string> ntpServers;
    ntpServers.push_back(ntpServer);
    ntpServers.push_back(backupNtpServer);
    tryNtpServerSync(backupNtpServer, ntpServers);
}
void NTPManager::tryNtpServerSync(const std::string &backupNtpServer,
                                   std::vector<std::string> &ntpServers) { // try to sync ntp with each server
    for (const auto &server : ntpServers) {
        try {
            synchronizeNTPWithModem(server);
            return;
        } catch (NTPException &e) {
            if (server == backupNtpServer) {
                esp_log_write(ESP_LOG_ERROR, TAG_TIMEMANAGER, "Failed to sync NTP with %s (%s)\n", server.c_str(),
                              e.what());
            }
            esp_log_write(ESP_LOG_ERROR, TAG_TIMEMANAGER, "Failed to sync NTP with %s (%s). Retrying with backup...\n",
                          server.c_str(), e.what());
        }
    }
}
void NTPManager::synchronizeNTPWithModem(const std::string &ntpServer) {
    esp_log_write(ESP_LOG_DEBUG, TAG_TIMEMANAGER, "NTP Server Syncing...\n");

    long error = modem.NTPServerSync(ntpServer.c_str(), timeZone);
    /**
      According to TinGsmNTP.tpp, the error codes are:
      case 1: "Network time synchronization is successful";
      case 61: "Network error";
      case 62: "DNS resolution error";
      case 63: "Connection error";
      case 64: "Service response error";
      case 65: "Service response timeout";
      default: "Unknown error: " + String(error);
     */
    // 255 is ok in our case (https://github.com/vshymanskyy/TinyGSM/pull/652)
    if (error != 1 && error != 255) {
        throw NTPException(modem.ShowNTPError(error).c_str());
    } else {
        esp_log_write(ESP_LOG_DEBUG, TAG_TIMEMANAGER, "NTP: Network time synchronization is successful\n");
    }
}

time_t NTPManager::timeToUnixEpochSeconds(const std::string &time, const std::string &format) {
    //	22/10/27,10:16:20+00
    struct tm tm {};
    time_t dateInEpoch = 0;

    // For some unknown reason, we have to use the C functions here instead of the std:: ones.
    if (strptime(time.c_str(), format.c_str(), &tm)) {
        time_t curTime;
        struct tm *timeinfo;

        timeinfo = localtime(&curTime);

        timeinfo->tm_year = tm.tm_year;
        timeinfo->tm_mon = tm.tm_mon;
        timeinfo->tm_mday = tm.tm_mday;
        timeinfo->tm_hour = tm.tm_hour;
        timeinfo->tm_min = tm.tm_min;
        timeinfo->tm_sec = tm.tm_sec;
        timeinfo->tm_isdst = -1;

        dateInEpoch = mktime(timeinfo);
    } else {
        throw StringToTimeConversionException("Failed to parse date");
    }
    return dateInEpoch;
}

void NTPManager::writeModemTimeToRTC() {
    auto gsmDateTimeString = modem.getGSMDateTime(DATE_FULL);
    esp_log_write(ESP_LOG_DEBUG, TAG_TIMEMANAGER, "GSM DateTime: %s\n", gsmDateTimeString.c_str());
    time_t time = timeToUnixEpochSeconds(gsmDateTimeString.c_str());

    rtc.setTime(time);
    esp_log_write(ESP_LOG_INFO, TAG_TIMEMANAGER, "Time set to EPOCH: %s\n", String(rtc.getEpoch()).c_str());
}
